%% getPolyIntsect
% Esta funo calcula as posies das intersees entre duas curvas 2D a
% partir de polinmios que as descrevem, dados em forma vetorial.
% Ex.: p(x) = x - 2x + 4x - 1 -> p = [1, -2, 4, -1]
%
% INPUT:
% * p1 -> vetor da primeira curva
% * p2 -> vetor da segunda curva
% OUTPUT:
% * intsects -> Coordenadas e valores dos polinmios nas intersees
%               [ x , p(x) ]
%
%% ------------------------------------------------------------------------
function intsects = getPolyIntsect(p1,p2)

% Obtm ordens dos polinmios
o1 = size(p1,2) - 1;
o2 = size(p2,2) - 1;

% Computa a maior ordem e identifica o polinmio de maior ordem
[order,whichPoly] = max([o1,o2]);

% Define vetor auxiliar para compatibilizar dimenses de p1 e p2
aux_p = zeros(1,order+1);

% Redefine o polinmio de menor ordem
switch whichPoly
    case 1
        aux_p(-o2+end:end) = p2;
        p2 = aux_p;
    case 2
        aux_p(-o1+end:end) = p1;
        p1 = aux_p;
end

% Descarta vetor auxiliar
clear aux_p

% Define o polinmio da diferena entre p1 e p2
p = p2 - p1;

% Calcula razes de p (ou seja, as coordenadas das intersees)
intsects = roots(p);

% Checa se existem intersees
if ~isempty(intsects)
    % Descarta intersees complexas
    stop = 0; % inicia 'flag' auxiliar
    n = 1; % inicia contador auxiiar
    while stop == 0
        % Checa se a coordenada da n-sima interseo  complexa
        if ~isreal(intsects(n))
            intsects(n) = []; % Deleta n-sima interseo do vetor intsects
            n = n - 1; % Atualiza o contador
        end
        % Checa se o contador chegou ao tamanho do vetor intsects
        if n == size(intsects,1) || isempty(intsects)
            stop = 1; % Sinaliza o fim do 'while'
        else
            n = n + 1; % Atualiza o contador
        end
    end
end

% Avalia o valor dos polinmios nas posices (x) das intersees
if ~isempty(intsects)
    intsects(:,2) = polyval(p1,intsects);
end
end